@@ -137,6 +137,10 @@ class Hist(object):
     def __iter__(self):
         """Return the contents of the bins."""
         return iter(self.y)
+    def scale(self, factor):
+        """Scale contents and errors by the given scale factor."""
+        self.y *= factor
+        self.yerr *= factor
     def show_titles(self):
         """Add the titles defined in the ROOT histogram to the figure."""
         plt.title(self.title)
