@@ -43,7 +43,10 @@ if '-h' not in sys.argv and len(sys.argv) > 1:
 ######## Define styles #######################################################
 
 default_styles=r"""
-## When using matplotlib, all text from the first column will be replaced
+##############################################################################
+######## Handling of LaTeX Expressions in matplotlib #########################
+
+## In the replace list below, all text from the first column will be replaced
 ## with the corresponding text in the second column for titles and labels;
 ## this is necessary to correctly transform ROOT's special LaTeX sytax into
 ## proper LaTeX sytax for use by matplotlib.
@@ -67,6 +70,21 @@ replace = [
     ('#'       , r''),
     ]
 
+##############################################################################
+######## Pretty File Names ###################################################
+
+## For more complex needs, you may want to specify a list of files along with
+## more descriptive names for the legend using the filenames option.  When
+## enabled, any filenames on the command line will be ignored.
+
+## filenames = [
+##     ('histTTbar.root', r'$\bar{t}t$'),
+##     ('histZmumu.root', r'$Z\rightarrow\mu\mu$'),
+##     ]
+
+##############################################################################
+######## Global Style Options ################################################
+
 colors = [
     ## a default set of contrasting colors the author happens to like
     ( 82, 124, 219), # blue
@@ -93,19 +111,54 @@ markers_root = [
      3, # asterisk
     ]
 
-## Define the size of the legend in ROOT
+#### Styles for --data
+data_color = (0,0,0)
+mc_color = (231, 139, 77)
+data_fmt = 'o'       # Choose marker for matplotlib
+data_marker = 4      # Choose marker for ROOT
+
+#### Titles produced by --area-normalize and --normalize
+area_normalized_title = "Fraction of Events in Bin"
+file_normalized_title = "Events Normalized to %s"
+scale_normalized_title = "Events Scaled by %d"
+
+#### Overflow and underflow text labels
+overflow_text = " Overflow"
+underflow_text = " Underflow"
+overflow_size = 'small'
+overflow_alpha = 0.5       # For matplotlib output
+
+#### Ratio plot
+ratio_fraction = 0.3
+
+##############################################################################
+######## ROOT Output #########################################################
+
+#### Define the size of the legend in ROOT
 legend_width = 0.22        # Fraction of canvas width
 legend_entry_height = 0.04 # Fraction of canvas height
 max_legend_height = 0.3    # Fraction of canvas height
+legend_left_bound = 0.20   # For left justification
+legend_right_bound = 0.87  # For right justification
+legend_upper_bound = 0.89  # For top justification
+legend_lower_bound = 0.15  # For bottom justification
+legend_codes = { 1 : 'upper right',
+                 2 : 'upper left',
+                 3 : 'lower left',
+                 4 : 'lower right',
+                 5 : 'right',
+                 6 : 'center left',
+                 7 : 'center right',
+                 8 : 'lower center',
+                 9 : 'upper center',
+                10 : 'center',
+                }
 
-## For more complex needs, you may want to specify a list of files along with
-## more descriptive names for the legend using the filenames option.  When
-## enabled, any filenames on the command line will be ignored.
+##############################################################################
+######## matplotlib Output ###################################################
+
+alpha = 0.5 # Used for bar, hist
 
-## filenames = [
-##     ('histTTbar.root', r'$\bar{t}t$'),
-##     ('histZmumu.root', r'$Z\rightarrow\mu\mu$'),
-##     ]
 """
 
 
@@ -209,13 +262,13 @@ def process_hist_root(path, name, counter):
         else:
             y_title = options.ylabel
     if options.area_normalize or (options.sticky and 'Norm' in name):
-        y_title = "Fraction of Events in Bin"
+        y_title = options.area_normalized_title
     if options.normalize:
         if type(options.normalize) == type(int()):
             file_name = files[options.normalize - 1].name
-            y_title = "Events Normalized to %s" % file_name
+            y_title = options.file_normalized_title % file_name
         else:
-            y_title = "Events Scaled by %d" % options.normalize
+            y_title = options.scale_normalized_title % options.normalize
     hists = []
     #### Apply options to hist from each file
     for i, file in enumerate(files):
@@ -234,9 +287,9 @@ def process_hist_root(path, name, counter):
         color = ROOT.TColor.GetColor(*options.colors[i][0:3])
         hist.SetLineColor(color)
         if options.fill or (options.data and (i+1) != options.data):
-            #hist.SetLineColor(ROOT.kBlack)
-            fill_color = color
-            hist.SetFillColor(fill_color)
+            color = ROOT.TColor.GetColor(*options.mc_color[0:3])
+            hist.SetLineColor(color)
+            hist.SetFillColor(color)
             hist.SetFillStyle(1001)
         if options.markers:
             hist.SetMarkerColor(color)
@@ -267,16 +320,32 @@ def process_hist_root(path, name, counter):
     stack = ROOT.THStack('st%.3i' % counter, title)
     legend_height = min(options.legend_entry_height * len(files) + 0.02,
                         options.max_legend_height)
-    if options.legend == 1:
-        legend_loc = [0.87 - options.legend_width, 0.89 - legend_height,
-                      0.87, 0.89]
-    elif options.legend == 8:
-        legend_loc = [0.22, 0.15, 0.90, 0.15 + legend_height]
-    elif options.legend == 'None':
-        legend_loc = [0,0,0,0]
+    if type(options.legend) == type(int()):
+        options.legend = options.legend_codes[options.legend]
+    elif options.legend.lower() == 'none':
+        options.legend = None
+    if options.legend:
+        if 'upper' in options.legend:
+            top = options.legend_upper_bound
+            bottom = options.legend_upper_bound - legend_height
+        elif 'lower' in options.legend:
+            bottom = options.legend_lower_bound
+            top = options.legend_lower_bound + legend_height
+        else:
+            top = 0.5 + legend_height / 2
+            bottom = 0.5 - legend_height / 2
+        if 'left' in options.legend:
+            left = options.legend_left_bound
+            right = options.legend_left_bound + options.legend_width
+        elif 'right' in options.legend:
+            right = options.legend_right_bound
+            left = options.legend_right_bound - options.legend_width
+        else:
+            right = 0.5 + options.legend_width / 2
+            left = 0.5 - options.legend_width / 2
+        legend_loc = [left, bottom, right, top]
     else:
-        print "Invalid legend parameter for ROOT output!"
-        sys.exit(1)
+        legend_loc = [0,0,0,0]
     legend = ROOT.TLegend(*legend_loc)
     for i, hist in enumerate(hists):
         if (i+1) != options.data:
@@ -298,9 +367,10 @@ def process_hist_root(path, name, counter):
         stack.SetMaximum(1.)
         stack.SetMinimum(0.)
     if options.data:
-        hists[options.data - 1].SetLineColor(ROOT.kBlack)
-        hists[options.data - 1].SetMarkerColor(ROOT.kBlack)
-        hists[options.data - 1].SetMarkerStyle(4) # circle
+        color = ROOT.TColor.GetColor(*options.data_color[0:3])
+        hists[options.data - 1].SetLineColor(color)
+        hists[options.data - 1].SetMarkerColor(color)
+        hists[options.data - 1].SetMarkerStyle(options.data_marker)
         hists[options.data - 1].SetMarkerSize(ROOT.gStyle.GetMarkerSize())
         if hists[options.data - 1].GetMaximum() > stack.GetMaximum():
             stack.SetMaximum(hists[options.data - 1].GetMaximum() * 1.2)
@@ -328,6 +398,7 @@ def process_hist_matplotlib(path, name, counter):
     plt.figure(1, figsize=(width, height))
     ax = plt.axes()
     ax.cla()
+    hists = []
     stack = r2m.HistStack()
     #### Apply options to hist from each file
     for i, file in enumerate(files):
@@ -347,10 +418,18 @@ def process_hist_matplotlib(path, name, counter):
                 ylabel += options.ylabel[1:]
             else:
                 ylabel = options.ylabel
+        if options.normalize:
+            if type(options.normalize) == type(int()):
+                file_name = files[options.normalize - 1].name
+                ylabel = options.file_normalized_title % file_name
+            else:
+                ylabel = options.scale_normalized_title % options.normalize
         hist = r2m.Hist(file.file.GetDirectory(path).Get(name),
-                        replace=options.replace,
+                        replace=options.replace, title=title,
                         xlabel=xlabel, ylabel=ylabel)
-        if not hist: continue
+        hists.append(hist)
+    for i, file in enumerate(files):
+        hist = hists[i]
         if options.efficiency_from:
             denom = r2m.Hist(file.file.Get(options.efficiency_from))
             eff, up, down = wilson([sum(hist.y)], [sum(denom.y)])
@@ -358,40 +437,62 @@ def process_hist_matplotlib(path, name, counter):
                            (eff[0] * 100., max(up[0], down[0]) * 100.))
             hist.y, up, down = wilson(hist.y, denom.y)
             hist.yerr = np.array([up, down])
+        if options.overflow or (options.sticky and 'Overflow' in name):
+            hist.y[-1] += hist.overflow
+        if options.underflow or (options.sticky and 'Underflow' in name):
+            hist.y[0] += hist.underflow
         if options.area_normalize or (options.sticky and 'Norm' in name):
             hist.y /= sum(hist.y)
+        if options.normalize:
+            if type(options.normalize) == type(int()):
+                hist.scale(sum(hists[options.normalize - 1]) / sum(hist.y))
+            else:
+                hist.scale(options.normalize)
         if options.markers: fmt = options.marker_styles[i]
         else:               fmt = 'o'
         if options.data and (i+1) == options.data:
             datahist = hist
             datahist.label = file.name
         else:
-            stack.add(hist, label=file.name, color=options.colors[i], fmt=fmt)
-    if (options.errorbar):
+            color = options.colors[i]
+            if options.data:
+                color = options.mc_color
+            stack.add(hist, label=file.name, color=color, fmt=fmt)
+    if options.errorbar:
         if options.data:
             try:
                 stack.hist(histtype='stepfilled')
             except ValueError:
                 pass
-            datahist.errorbar(yerr=True, fmt='o', color='k')
+            datahist.errorbar(yerr=True, fmt=options.data_fmt,
+                              color=options.data_color)
         else:
             plots = stack.errorbar(yerr=True)
     if options.stack:
         stack.barstack()
     if options.bar:
-        stack.bar(alpha=0.5)
+        stack.bar(alpha=options.alpha)
     if options.hist:
         try:
-            stack.hist(alpha=0.5, histtype='stepfilled')
+            stack.hist(alpha=options.alpha, histtype='stepfilled')
         except ValueError:
             pass
     if options.efficiency:
         plt.ylim(0, 1)
+    plt.xlim(hist.xedges[0], hist.xedges[-1])
     if options.logx: ax.set_xscale('log')
     if options.logy: ax.set_yscale('log')
     if options.numbering:
         plt.text(0.98, 0.98, counter, transform=plt.figure(1).transFigure,
                  size="small", ha='right', va='top')
+    if options.overflow or (options.sticky and 'Overflow' in name):
+        plt.text(hist.x[-1], plt.ylim()[0], options.overflow_text,
+                 rotation='vertical', ha='center',
+                 alpha=options.overflow_alpha, size=options.overflow_size)
+    if options.underflow or (options.sticky and 'Underflow' in name):
+        plt.text(hist.x[0], plt.ylim()[0], options.underflow_text,
+                 rotation='vertical', ha='center',
+                 alpha=options.overflow_alpha, size=options.overflow_size)
     stack.show_titles()
     if options.legend != 'None':
         plt.legend(numpoints=1, loc=options.legend)
@@ -418,7 +519,8 @@ def save_plot(stack, plot_dir, path, name, counter):
 def report_progress(counter, divisor=1):
     """Print the current number of finished plots."""
     if counter % divisor == 0:
-        print "\r%i plots written to %s/" % (counter, options.output),
+        print "\r%i plots of %i written to %s/" % (counter, options.nplots,
+                                                   options.output),
         sys.stdout.flush()
 
 def merge_pdf():
@@ -470,7 +572,7 @@ def add_ratio_plot(hists, stack, counter):
     """Divide canvas into two parts, and plot the ratio on the bottom."""
     ## Both pads are set to the full canvas size to maintain font sizes
     ## Fill style 4000 used to ensure pad transparency because of this
-    div = 0.3 # portion of canvas to use for ratio plot
+    div = options.ratio_fraction
     margins = [ROOT.gStyle.GetPadTopMargin(), ROOT.gStyle.GetPadBottomMargin()]
     useable_height = 1 - (margins[0] + margins[1])
     canvas.Clear()
@@ -615,7 +717,7 @@ Function: overlays corresponding histograms from several files, dumping the
 
 Power users: advanced operation using configuration files is described in the
   full online documentation.  This allows you control over colors, styles,
-  names for the legends, and more.
+  names for the legends, and more.  Get the default config with --config.
 
 Matplotlib: if you have the matplotlib python plotting library installed on your
   system, you can produce output in matplotlib.  By activating any of the
@@ -625,7 +727,7 @@ Matplotlib: if you have the matplotlib python plotting library installed on your
     parser = optparse.OptionParser(usage=usage, 
                                    version='%s %s' % ('%prog', __version__))
     parser.add_option('--config', action="store_true",
-                      help="Do nothing but write a template configuration file "
+                      help="do nothing but write a template configuration file "
                       "called overlayHists_config.py")
     parser.add_option('-e', '--ext', default='pdf', 
                       help="choose an output extension; default is pdf")
@@ -659,9 +761,8 @@ Matplotlib: if you have the matplotlib python plotting library installed on your
                       "colormap")
     parser.add_option('--legend', default=1, metavar="LOC",
                       help="Place legend in LOC, according to matplotlib "
-                      "location codes; for ROOT, you can use 1 (default) for "
-                      "upper right or 8 for lower center; "
-                      "use 'None' to not show the legend")
+                      "location codes; examples include 'upper right', "
+                      "'center', or 'center left'")
     parser.add_option('--title', default=None,
                       help="Replace the plot titles, or add to them by "
                       "preceeding with a '+'")
@@ -681,8 +782,8 @@ Matplotlib: if you have the matplotlib python plotting library installed on your
         "you can use '-s' "
         "to turn on sticky keywords (such as 'Norm').  Any plot that includes "
         "the given keyword in its ROOT name will have the option applied "
-        "regardless of its presence or absence on the command line."
-        "The last three options are currently only available in ROOT output."
+        "regardless of its presence or absence on the command line.  "
+        "'ratio' is currently only available in ROOT output."
         )
     group1.add_option('-n', '--area-normalize', action='store_true',
                       default=False,
@@ -763,6 +864,7 @@ Matplotlib: if you have the matplotlib python plotting library installed on your
     options = process_options(options)
     ## here, we decend into the files to start plotting
     plots_to_make = process_directory('', files)
+    options.nplots = len(plots_to_make)
     if use_multiprocessing:
         queue = multiprocessing.JoinableQueue()
         for args in plots_to_make:
@@ -776,7 +878,8 @@ Matplotlib: if you have the matplotlib python plotting library installed on your
     else:
         for args in plots_to_make:
             process_hist(*args)
-    print ''
+    print "\r%i plots of %i written to %s/" % (options.nplots, options.nplots,
+                                               options.output)
     if options.ext == 'pdf':
         merge_pdf()
 
